;------------------------------------------------------------------------------
; usbconv for AT90S2313  Ver 1.0  2005.03.10 by K.Ishikawa
;------------------------------------------------------------------------------

	.include "tn2313def.inc"

;------------------------------------------------------------------------------
;	AVR Port Assign List
;
;	PortB bit0  LED
;	      bit1  Select 0:Mouse 1:Keyboard 
;	      bit2  
;	      bit3  
;	      bit4  
;	      bit5  (MOSI)
;	      bit6  (MISO)
;	      bit7  (SCK)
;	PortD bit0  RxD
;	      bit1  TxD
;	      bit2  USB BUS D+
;	      bit3  USB BUS D-
;	      bit4  
;	      bit5  
;	      bit6  

;------------------------------------------------------------------------------

	.def	reg_0		= r10
	.def	reg_1		= r11
	.def	reg_2		= r12
	.def	reg_3		= r13
	.def	crc_temp	= r14
	.def	rc_state	= r15
	.def	rc_num		= r16
	.def	p3do_cnt	= r17
	.def	p3do_bit	= r18
	.def	uart_err	= r19
	.def	uart_h		= r20
	.def	uart_data	= r21
	.def	arg1		= r22
	.def	arg2		= r23
	.def	tmp1		= r24
	.def	tmp2		= r25

	.equ    RECEIVING		= $60	; Start UART Buffer Location
	.equ	DEVICE1	=	$70
	.equ	DEVICE2	=	$80
	.equ	DEVICE3	=	$90
	.equ	DEVICE4	=	$a0


;------------------------------------------------------------------------------


;------------------------------------------------------------------------------

	rjmp	MAIN		; Reset Handler
	rjmp	EXT_INT0	; IRQ0 Handler
	rjmp	EXT_INT1	; IRQ0 Handler
	rjmp	TIM1_CAPT	; Timer1 Capture Handler
	rjmp	TIM1_COMP	; Timer1 Compare Handler
	rjmp	TIM1_OVF	; Timer1 Overflow Handler
	rjmp	TIM0_OVF	; Timer0 Overflow Handler
	rjmp	USART_Receive	; UART RX Complete Handler
	rjmp	UART_DRE	; UDR Empty Handler
	rjmp	UART_TXC	; UART TX Complete Handler
	rjmp	ANA_COMP	; Analog Comparator Handler

;------------------------------------------------------------------------------

EXT_INT0:
EXT_INT1:
TIM1_CAPT:
TIM1_COMP:
TIM1_OVF:
TIM0_OVF:
UART_RXC:
UART_DRE:
UART_TXC:
ANA_COMP:
	reti


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

	.org	0x100

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

MAIN:
	; **** Initialize Stack Pointer ****
	ldi		tmp1, LOW(RAMEND)	; Stack Pointer Setup
	out		SPL, tmp1			; Stack Pointer Low Byte	
	; **** Turn off Watchdog timer ****
	ldi		tmp1, 0x18			; 
	out		WDTCR, tmp1			; 
	ldi		tmp1, 0x10			; 
	out		WDTCR, tmp1			;
	; **** Initialize Comparator ****
	ldi		tmp1, 0x80			; Disable comparator to save power
	out		ACSR, tmp1			; 
	; **** Initialize I/O Port ****
	ldi		tmp1, 0x05			; Set PB2 (3do_in) as Output Mode
	out		DDRB, tmp1
	ldi		tmp1, 0xff			; IN H
	out		PORTB, tmp1
	ldi		tmp1, 0x02			; Set PD0-6 as Input Mode
	out		DDRD, tmp1
	ldi		tmp1, 0x00			; PD2-3 HI-Z
	out		PORTD, tmp1
	; **** Initialize UART ****
	ldi		tmp1, 0
	out		UBRRH, tmp1
	ldi		tmp1, 1
	out		UBRRL, tmp1			; UART Baud rate generator
	ldi		tmp1, 0xe0
	out		UCSRA, tmp1
	ldi		tmp1, 0x94			
	out		UCSRB, tmp1			
	ldi		tmp1, 0x0e			
	out		UCSRC, tmp1
	; **** Initialize Register ****
	ldi		tmp1, 0
	mov		reg_0, tmp1
	ldi		tmp1, 1
	mov		reg_1, tmp1
	ldi		tmp1, 2
	mov		reg_2, tmp1
	ldi		tmp1, 3
	mov		reg_3, tmp1
	ldi		tmp1, 2
	mov		rc_state, tmp1
	ldi		rc_num, 0
	;*******Init Dev**********
	ldi		tmp1, 0
	ldi		ZH, HIGH(DEVICE1)
	ldi		ZL, LOW(DEVICE1)
	st		Z, tmp1
	std		Z+0x10, tmp1
	std		Z+0x20, tmp1
	std		Z+0x30, tmp1
	; **** Power On Wait ****
	rcall	DELAY_1S
	sei							; Enable Global Interrupt

	; **** Main Loop ****
	rjmp	MAIN_LOOP


;------------------------------------------------------------------------------
USART_proc:
	ldi		YH, HIGH(RECEIVING)
	ldi		YL, LOW(RECEIVING)
	mov		rc_num, uart_data
	mov		crc_temp, uart_data
	clr		rc_state
	cpi		rc_num, 21
	brsh	USART_ReceiveErrorSize
	out		SREG, uart_err	
	reti
USART_ReceiveErrorSize:
	ldi		rc_num, 1
	mov		rc_state, rc_num	
	inc		rc_state
	out		SREG, uart_err
	reti

USART_Receive:
; Get status and 9th bit, then data from buffer
	in		uart_err, SREG
	in 		uart_h, UCSRB
	in		uart_data, UDR

	andi	uart_h, 2
	breq	USART_proc

USART_data:
	cp		rc_state, rc_num
	breq	USART_ReceiveFinish
	brsh	USART_ReceiveEnd	

	eor		crc_temp, uart_data
	bst		crc_temp, 7
	lsl		crc_temp
	bld		crc_temp, 0
	st		Y+, uart_data
	inc		rc_state

	out		SREG, uart_err
	reti

USART_ReceiveFinish:
	cp		uart_data, crc_temp
	brne	USART_ReceiveEnd

	//send ACK!

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	ldi		XH, HIGH(RECEIVING)
	ldi		XL, LOW(RECEIVING)
	ldi		YH, HIGH(DEVICE1)
	ldi		YL, LOW(DEVICE1)
	ld		crc_temp, X+
	mov		uart_data, crc_temp

	lsr		uart_data
	lsr		uart_data
	andi	uart_data, 0x30
	add		YL, uart_data

	mov		uart_data, crc_temp
	andi	uart_data, 0x7
	ld		uart_h, Y
	andi	uart_h, 0x7
	cp		uart_h, uart_data
	breq	USART_ReceiveError		

	mov		uart_data, crc_temp
	andi	uart_data, 0x38
	breq	USART_COPY_MOUSE
	cpi		uart_data, 0x08
	breq	USART_COPY_JOY
	clr		uart_data
	st		Y, uart_data
	rjmp	USART_ReceiveError

USART_COPY_JOY:
	ldi		uart_data, 0x80
	or		crc_temp, uart_data
	st		Y+, crc_temp
	ldi		uart_data, 2
	st		Y+, uart_data

	ld		uart_data, X+
	st		Y+, uart_data
	ld		uart_data, X+
	st		Y+, uart_data

	rjmp	USART_ReceiveError

USART_COPY_MOUSE:
	ldi		uart_data, 0x80
	or		crc_temp, uart_data
	st		Y+, crc_temp
	ldi		uart_data, 4
	st		Y+, uart_data
	ldi		uart_data, 0x49
	st		Y+, uart_data
	ld		uart_data, X+
	st		Y+, uart_data
	ld		uart_data, X+
	st		Y+, uart_data
	ld		uart_data, X+
	st		Y+, uart_data


USART_ReceiveError:

	mov		rc_state, rc_num	
	inc		rc_state	
USART_ReceiveEnd:	
	out		SREG, uart_err
	reti



//////////////delay///////////////////
DELAY_1S:
	ldi	tmp1, 0x80
DELAY_1:
	ldi	tmp2, 0x80
DELAY_2:
	ldi arg2, 0x40
DELAY_3:
	dec arg2
	brne	DELAY_3
	dec tmp2
	brne	DELAY_2
	dec	tmp1
	brne	DELAY_1
	ret

//////////////////////SENDDEVICE///////////////////


MAIN_LOOP:



JP_ENTERPOINT:
	clr		p3do_cnt
JP_SEARCH_UPSTART:
	sbis	PINB, 4
	rjmp	JP_STEP_OUT
	inc		p3do_cnt
	cpi		p3do_cnt, 0x10
	brne	JP_SEARCH_UPSTART
JP_WAIT_DOWN:
	sbic	PINB, 4
	rjmp	JP_WAIT_DOWN
	cli

	sbi		PORTD, 1

	ldi		ZH, HIGH(DEVICE1)	
	ldi		ZL, LOW(DEVICE1)
	ld		arg1, Z+
	sbrc	arg1, 7
	rcall	SEND_DEVICE
	ldi		ZH, HIGH(DEVICE1)	
	ldi		ZL, LOW(DEVICE1)
	rcall	CLEAR_DEVICE


	ldi		ZH, HIGH(DEVICE2)	
	ldi		ZL, LOW(DEVICE2)
	ld		arg1, Z+
	sbrc	arg1, 7
	rcall	SEND_DEVICE
	ldi		ZH, HIGH(DEVICE2)	
	ldi		ZL, LOW(DEVICE2)
	rcall	CLEAR_DEVICE


	ldi		ZH, HIGH(DEVICE3)	
	ldi		ZL, LOW(DEVICE3)
	ld		arg1, Z+
	sbrc	arg1, 7
	rcall	SEND_DEVICE
	ldi		ZH, HIGH(DEVICE3)	
	ldi		ZL, LOW(DEVICE3)
	rcall	CLEAR_DEVICE


	ldi		ZH, HIGH(DEVICE4)	
	ldi		ZL, LOW(DEVICE4)
	ld		arg1, Z+
	sbrc	arg1, 7
	rcall	SEND_DEVICE
	ldi		ZH, HIGH(DEVICE4)	
	ldi		ZL, LOW(DEVICE4)
	rcall	CLEAR_DEVICE


JP_WAIT_DOWN3:
	sbic	PINB, 4
	rjmp	JP_WAIT_DOWN3

	sbi		PORTB, 2
	cbi		PORTD, 1

	sei
JP_STEP_OUT:	
	
	rjmp	MAIN_LOOP




;-------------------------------------------

SEND_DEVICE:

JP_WAIT_DOWN2:
	sbic	PINB, 4
	rjmp	JP_WAIT_DOWN2

	ld		p3do_cnt, Z+
	ld		arg1, Z+
	ldi		p3do_bit, 0x8

JP_NEXT_STEP0:	
	sbrs	arg1, 7
	cbi		PORTB, 2
	sbrc	arg1, 7
	sbi		PORTB, 2
	lsl		arg1
	dec		p3do_bit
	brne	JP_NEXT_STEP1
	ldi		p3do_bit, 0x8

	dec		p3do_cnt
	breq	JP_PROCESS_END
	ld		arg1, Z+

JP_NEXT_STEP1:
	sbis	PINB, 4
	rjmp	JP_NEXT_STEP1
		
JP_NEXT_STEP2:
	sbic	PINB, 4
	rjmp	JP_NEXT_STEP2

	rjmp	JP_NEXT_STEP0

JP_PROCESS_END:

JP_WAIT_UP:
	sbis	PINB, 4
	rjmp	JP_WAIT_UP

	ret	
		
CLEAR_DEVICE:
	ld		arg1, Z
	andi	arg1, 0x38
	brne	CLEAR_FIN
	ldd		arg1, Z+3
	andi	arg1, 0xf0
	std		Z+3, arg1
	clr		arg1
	std		Z+4, arg1
	std		Z+5, arg1

CLEAR_FIN:
	ret
